.text-truncate.text-right{text-align:left;direction:rtl}.text-truncate.text-right:focus{direction:ltr}.text-truncate.text-right[value=""]:focus{text-align:left;direction:rtl}.table-xy-overflow-scroll{display:block;width:100%;max-height:400px;overflow-x:auto;overflow-y:auto}.breadcrumb{background-color:transparent;padding:0}.breadcrumb-link{width:100%;overflow:hidden;white-space:nowrap;background-color:inherit}.breadcrumb-item:before{vertical-align:top}.log-visualizer-legend{max-width:300px;overflow:auto}.log-visualizer-tooltip{max-width:300px}.plot-hidden{min-width:800px;position:absolute;left:-9999px}.ReactTable{display:table}.form-check-label-break-word{width:100%;word-wrap:break-word;overflow-wrap:break-word}.result-highlight{background-color:#fffbcc}.recharts-active-dot{pointer-events:none}.recharts-tooltip-wrapper{z-index:10000}