<!DOCTYPE html>
<html lang='en'>
  <head>
    <meta charset='utf-8'>
    <meta name='viewport' content='width=device-width, initial-scale=1, shrink-to-fit=no'>
    <title>ChainerUI</title>
    <link rel="icon" href="/favicon.ico">
    <link rel="stylesheet" href="{{ url_for('static', filename='dist/vendor.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='dist/chainerui.css') }}">
    <script type="text/javascript" src="{{ url_for('static', filename='dist/vendor.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='dist/chainerui.js') }}"></script>
  </head>

  <body>
    <div id="chainerui-root"></div>
  </body>
</html>
