from chainerui import _version
from chainerui.client.client import init  # NOQA
from chainerui.client.client import log  # NOQA
from chainerui.client.client import log_reporter  # NOQA


__version__ = _version.__version__
